Clazz.declarePackage ("J.adapter.writers");
Clazz.load (["J.api.JmolWriter"], "J.adapter.writers.PWMATWriter", ["java.lang.Float", "JU.AU", "$.P3", "$.PT", "JU.Logger"], function () {
c$ = Clazz.decorateAsClass (function () {
this.vwr = null;
this.oc = null;
this.uc = null;
this.names = null;
this.bs = null;
Clazz.instantialize (this, arguments);
}, J.adapter.writers, "PWMATWriter", null, J.api.JmolWriter);
Clazz.makeConstructor (c$, 
function () {
});
Clazz.overrideMethod (c$, "set", 
function (viewer, oc, data) {
this.vwr = viewer;
this.oc = (oc == null ? this.vwr.getOutputChannel (null, null) : oc);
}, "JV.Viewer,JU.OC,~A");
Clazz.overrideMethod (c$, "write", 
function (bs) {
this.bs = (bs == null ? this.vwr.bsA () : bs);
try {
var n = bs.cardinality ();
var line = JU.PT.formatStringI ("%12i\n", "i", n);
this.oc.append (line);
this.uc = this.vwr.ms.getUnitCellForAtom (bs.nextSetBit (0));
this.names = this.vwr.getDataObj ("*", bs, 1);
this.writeLattice ();
this.writePositions ();
this.writeDataBlocks ();
} catch (e) {
if (Clazz.exceptionOf (e, Exception)) {
System.err.println ("Error writing PWmat file " + e);
} else {
throw e;
}
}
return this.toString ();
}, "JU.BS");
Clazz.defineMethod (c$, "writeLattice", 
 function () {
this.oc.append ("Lattice vector\n");
if (this.uc == null) {
this.uc = this.vwr.getSymTemp ();
var bb = this.vwr.getBoundBoxCornerVector ();
var len = Math.round (bb.length () * 2);
this.uc.setUnitCell ( Clazz.newFloatArray (-1, [len, len, len, 90, 90, 90]), false);
}var abc = this.uc.getUnitCellVectors ();
var f = "%18.10p%18.10p%18.10p\n";
this.oc.append (JU.PT.sprintf (f, "p",  Clazz.newArray (-1, [abc[1]])));
this.oc.append (JU.PT.sprintf (f, "p",  Clazz.newArray (-1, [abc[2]])));
this.oc.append (JU.PT.sprintf (f, "p",  Clazz.newArray (-1, [abc[3]])));
JU.Logger.info ("PWMATWriter: LATTICE VECTORS");
});
Clazz.defineMethod (c$, "writePositions", 
 function () {
var cx = this.getData ("CONSTRAINTS_X");
var cy = (cx == null ? null : this.getData ("CONSTRAINTS_Y"));
var cz = (cy == null ? null : this.getData ("CONSTRAINTS_Z"));
this.oc.append ("Position, move_x, move_y, move_z\n");
var f = "%4i%18.12p%18.12p%18.12p" + (cz == null ? "  1  1  1" : "%4i%4i%4i") + "\n";
var a = this.vwr.ms.at;
var p =  new JU.P3 ();
for (var ic = 0, i = this.bs.nextSetBit (0); i >= 0; i = this.bs.nextSetBit (i + 1), ic++) {
p.setT (a[i]);
this.uc.toFractional (p, true);
if (cz == null) {
this.oc.append (JU.PT.sprintf (f, "ip",  Clazz.newArray (-1, [Integer.$valueOf (a[i].getElementNumber ()), p])));
} else {
var ix = Clazz.floatToInt (cx[ic]);
var iy = Clazz.floatToInt (cy[ic]);
var iz = Clazz.floatToInt (cz[ic]);
this.oc.append (JU.PT.sprintf (f, "ipiii",  Clazz.newArray (-1, [Integer.$valueOf (a[i].getElementNumber ()), p, Integer.$valueOf (ix), Integer.$valueOf (iy), Integer.$valueOf (iz)])));
}}
JU.Logger.info ("PWMATWriter: POSITIONS");
});
Clazz.defineMethod (c$, "getData", 
 function (name) {
name = "property_pwm_" + name.toLowerCase ();
for (var i = this.names.length; --i >= 0; ) {
if (this.names[i] != null && name.equalsIgnoreCase (this.names[i])) {
this.names[i] = null;
return this.vwr.getDataObj (name, this.bs, 1);
}}
return null;
}, "~S");
Clazz.defineMethod (c$, "getVectors", 
 function (name) {
var vectors = JU.AU.newFloat2 (3);
var x = vectors[0] = this.getData (name + "_x");
x = vectors[1] = (x == null ? null : this.getData (name + "_y"));
vectors[2] = (x == null ? null : this.getData (name + "_z"));
return (vectors[2] == null ? null : vectors);
}, "~S");
Clazz.defineMethod (c$, "writeDataBlocks", 
 function () {
this.writeVectors ("FORCE");
this.writeVectors ("VELOCITY");
this.writeMagnetic ();
this.writeMoreData ();
});
Clazz.defineMethod (c$, "writeVectors", 
 function (name) {
var xyz = this.getVectors (name);
if (xyz[0] == null) return;
var a = this.vwr.ms.at;
var p =  new JU.P3 ();
this.oc.append (name.toUpperCase ()).append ("\n");
var f = "%4i%18.12p%18.12p%18.12p\n";
for (var ic = 0, i = this.bs.nextSetBit (0); i >= 0; i = this.bs.nextSetBit (i + 1), ic++) {
p.set (xyz[0][ic], xyz[1][ic], xyz[2][ic]);
this.oc.append (JU.PT.sprintf (f, "ip",  Clazz.newArray (-1, [Integer.$valueOf (a[i].getElementNumber ()), p])));
}
}, "~S");
Clazz.defineMethod (c$, "writeMagnetic", 
 function () {
var m = this.writeItems ("MAGNETIC");
if (m == null) return;
this.writeItem2 (m, "CONSTRAINT_MAG");
this.writeVectors ("MAGNETIC_XYZ");
});
Clazz.defineMethod (c$, "writeItem2", 
 function (m, name) {
var v = this.getData (name);
if (v == null) return;
var a = this.vwr.ms.at;
this.oc.append (name.toUpperCase ()).append ("\n");
var f = "%4i%18.12f%18.12f\n";
for (var ic = 0, i = this.bs.nextSetBit (0); i >= 0; i = this.bs.nextSetBit (i + 1), ic++) {
this.oc.append (JU.PT.sprintf (f, "iff",  Clazz.newArray (-1, [Integer.$valueOf (a[i].getElementNumber ()), Float.$valueOf (m[ic]), Float.$valueOf (v[ic])])));
}
}, "~A,~S");
Clazz.defineMethod (c$, "writeItems", 
 function (name) {
var m = this.getData (name);
if (m == null) return null;
var a = this.vwr.ms.at;
this.oc.append (name.toUpperCase ()).append ("\n");
var f = "%4i%18.12f\n";
for (var ic = 0, i = this.bs.nextSetBit (0); i >= 0; i = this.bs.nextSetBit (i + 1), ic++) {
this.oc.append (JU.PT.sprintf (f, "if",  Clazz.newArray (-1, [Integer.$valueOf (a[i].getElementNumber ()), Float.$valueOf (m[ic])])));
}
return m;
}, "~S");
Clazz.defineMethod (c$, "writeMoreData", 
 function () {
});
Clazz.overrideMethod (c$, "toString", 
function () {
return (this.oc == null ? "" : this.oc.toString ());
});
});
